"""CLI info command."""
from argparse import ArgumentParser, Namespace
from typing import List, Tuple

from InquirerPy.utils import color_print  # type: ignore
from InquirerPy.separator import Separator  # type: ignore
from halo.halo import Halo  # type: ignore

from redbrick.cli.input.text import CLIInputText
from redbrick.cli.input.uuid import CLIInputUUID
from redbrick.cli.project import CLIProject
from redbrick.cli.cli_base import CLIInfoInterface


class CLIInfoController(CLIInfoInterface):
    """CLI info command controller."""

    def __init__(self, parser: ArgumentParser) -> None:
        """Intialize info sub commands."""
        parser.add_argument(
            "--get",
            "-g",
            choices=[self.SETTING_LABELSTORAGE],
            help="Get a project setting info",
        )
        parser.add_argument(
            "--set",
            "-s",
            choices=[self.SETTING_LABELSTORAGE],
            help="Set a project setting",
        )
        parser.add_argument(
            "path",
            nargs="?",
            default=".",
            help="Path of project (Default: current directory)",
        )

    def handler(self, args: Namespace) -> None:
        """Handle info command."""
        self.args = args
        project = CLIProject.from_path(path=self.args.path)
        assert project, "Not a valid project"
        self.project = project

        if self.args.get:
            self.handle_get()
        elif self.args.set:
            self.handle_set()
        else:
            self.handle_info()

    @staticmethod
    def _color_print_info(title: str, attrs: List[Tuple[str, str]]) -> None:
        text = [("green", f"{Separator()} {title} {Separator()}\n")]
        for key, value in attrs:
            text.append(("", f"{key}: {value}\n"))
        color_print(text)

    def handle_get(self) -> None:
        """Handle get sub command."""
        if self.args.get == self.SETTING_LABELSTORAGE:
            with Halo(text="Get: Label Storage", spinner="dots") as spinner:
                try:
                    storage_id, path = self.project.project.label_storage
                except Exception as error:
                    spinner.fail()
                    raise error
            spinner.succeed()

            CLIInfoController._color_print_info(
                "Label Storage", [("Storage ID", storage_id), ("Path prefix", path)]
            )

    def handle_set(self) -> None:
        """Handle set sub command."""
        if self.args.set == self.SETTING_LABELSTORAGE:
            storage_id = CLIInputUUID(None, "Storage ID").get()
            path = CLIInputText(None, "Path prefix").get()
            with Halo(text="Set: Label Storage", spinner="dots") as spinner:
                try:
                    new_storage_id, new_path = self.project.project.set_label_storage(
                        storage_id, path
                    )
                except Exception as error:
                    spinner.fail()
                    raise error
            spinner.succeed()

            CLIInfoController._color_print_info(
                "Label Storage",
                [("Storage ID", new_storage_id), ("Path prefix", new_path)],
            )

    def handle_info(self) -> None:
        """Handle empty sub command."""
        org = self.project.org
        project = self.project.project

        org_data, project_data = [], []
        org_data.append(("ID", org.org_id))
        org_data.append(("Name", org.name))

        project_data.append(("ID", project.project_id))
        project_data.append(("Name", project.name))
        project_data.append(("Type", project.project_type.value))
        project_data.append(("Taxonomy", project.taxonomy_name))
        project_data.append(("URL", project.url))

        CLIInfoController._color_print_info("Organization", org_data)
        CLIInfoController._color_print_info("Project", project_data)



"""CLI prune command."""
import os
import re
from datetime import datetime
from argparse import ArgumentParser, Namespace

from redbrick.cli.project import CLIProject
from redbrick.cli.cli_base import CLIExportInterface, CLIPruneInterface
from redbrick.utils.logging import print_info, print_warning


class CLIPruneController(CLIPruneInterface):
    """CLI prune command controller."""

    def __init__(self, parser: ArgumentParser) -> None:
        """Intialize prune sub commands."""
        parser.add_argument(
            "--time",
            "-t",
            type=int,
            default=1,
            help="Time (Default: 1)",
        )
        parser.add_argument(
            "--unit",
            "-u",
            choices=["s", "m", "h", "d"],
            default="d",
            help="Unit [s(seconds), m(minutes), h(hours), d(days)] (Default: d)",
        )
        parser.add_argument(
            "path",
            nargs="?",
            default=".",
            help="Directory containing exported files (Default: current directory)",
        )

    def handler(self, args: Namespace) -> None:
        """Handle prune command."""
        self.args = args
        project = CLIProject.from_path()
        assert project, "Not a valid project"
        self.project = project

        self.handle_prune()

    def handle_prune(self) -> None:
        """Handle empty sub command."""
        directory = self.args.path
        if not os.path.isdir(directory):
            print_warning(f"Directory {directory} does not exist")
            return

        pattern = re.compile(
            r"export_("
            + (
                "|".join(
                    (CLIExportInterface.FORMAT_REDBRICK, CLIExportInterface.FORMAT_COCO)
                )
            )
            + r")_[^.]+\.json$"
        )
        threshold = datetime.now().timestamp() - (
            self.args.time * {"s": 1, "m": 60, "h": 3600, "d": 86400}[self.args.unit]
        )
        print_info(
            f"Deleting files that were last modified before {datetime.fromtimestamp(threshold)}"
        )
        for item in os.listdir(directory):
            path = os.path.join(directory, item)
            if (
                re.match(pattern, item)
                and os.path.isfile(path)
                and os.stat(path).st_mtime < threshold
            ):
                os.remove(path)
                print_info(f"Deleted: {path}")



"""Public interface to learning module."""
import os
import json
import asyncio
from typing import Dict, List, Optional, Tuple, Callable, Any, Union, TypeVar, cast
from functools import partial, wraps
from copy import deepcopy

import aiohttp
import tqdm

from redbrick.common.constants import MAX_CONCURRENCY
from redbrick.utils.files import NIFTI_FILE_TYPES, upload_files
from redbrick.common.context import RBContext
from redbrick.common.enums import TaskStates
from redbrick.utils.pagination import PaginationIterator
from redbrick.utils.rb_label_utils import clean_rb_label
from redbrick.utils.async_utils import gather_with_concurrency
from redbrick.utils.logging import (
    print_error,
    print_info,
    print_warning,
    handle_exception,
)

Func = TypeVar("Func", bound=Callable[..., Any])


def check_learning_stage(func: Func) -> Func:
    """Implement decorator to check if active learning is enabled for current project."""

    @wraps(func)
    def wrap(obj: Union["Learning", "Learning2"], *args: Any, **kwargs: Any) -> Any:
        if not obj.stage_name:
            raise Exception("No stage available for active learning in this project.")
        return func(obj, *args, **kwargs)

    return cast(Func, wrap)


class Learning:
    """Perform active learning and upload the results with the RedBrick API."""

    def __init__(
        self, context: RBContext, org_id: str, project_id: str, stage_name: str = ""
    ) -> None:
        """Construct Learning module."""
        self.context = context
        self.org_id = org_id
        self.project_id = project_id
        self.stage_name = stage_name

    @handle_exception
    @check_learning_stage
    def get_learning_info(self) -> Dict:
        """
        Get a dictionary with lightly parsed redbrick response.

        The dictonary has fields:
            labeled
            unlabeled
            taxonomy
            cycle
        """
        cycle = self.context.learning.check_for_job(
            self.org_id, self.project_id, self.stage_name
        )
        if not cycle:
            raise Exception("No available jobs")

        my_iter = PaginationIterator(
            partial(
                self.context.learning.get_batch_of_tasks,
                self.org_id,
                self.project_id,
                self.stage_name,
                100,
            )
        )
        taxonomy, td_type = self.context.learning.get_taxonomy_and_type(
            self.org_id, self.project_id, self.stage_name
        )

        def _parse_labeled_entry(item: Dict) -> Dict:
            datapoint = item.get("datapoint", {}) or {}
            items_presigned = datapoint.get("itemsPresigned", []) or []
            name = datapoint.get("name")
            items = datapoint.get("items", []) or []
            task_id = item.get("taskId")
            groundtruth = item.get("groundTruth", {}) or {}
            labels = [
                clean_rb_label(label)
                for label in json.loads(groundtruth.get("labelsData", "[]") or "[]")
            ]

            return {
                "taskId": task_id,
                "items": items,
                "itemsPresigned": items_presigned,
                "name": name,
                "labels": labels,
            }

        def _parse_unlabeled_entry(item: Dict) -> Dict:
            datapoint = item.get("datapoint", {}) or {}
            items_presigned = datapoint.get("itemsPresigned", []) or []
            name = datapoint.get("name")
            items = datapoint.get("items", []) or []
            task_id = item.get("taskId")
            return {
                "taskId": task_id,
                "items": items,
                "itemsPresigned": items_presigned,
                "name": name,
            }

        labeled: List[Dict] = []
        unlabeled: List[Dict] = []

        print_info("Downloading tasks for active learning")
        for entry in tqdm.tqdm(my_iter, unit=" tasks"):
            if entry.get("groundTruth"):
                labeled.append(_parse_labeled_entry(entry))
            else:
                unlabeled.append(_parse_unlabeled_entry(entry))

        return {
            "labeled": labeled,
            "unlabeled": unlabeled,
            "taxonomy": taxonomy,
            "cycle": cycle,
            "type": td_type,
        }

    async def _update_task(
        self, session: aiohttp.ClientSession, cycle: int, task: Dict
    ) -> Optional[Dict]:
        """Attempt to update task."""
        try:
            await self.context.learning.send_batch_learning_results_async(
                session,
                self.org_id,
                self.project_id,
                self.stage_name,
                cycle,
                [task],
            )
        except ValueError as error:
            print_error(error)
            point_error = deepcopy(task)
            point_error["error"] = error
            return point_error
        return None

    async def _update_tasks(self, cycle: int, tasks: List[Dict]) -> List[Dict]:
        failed = []
        conn = aiohttp.TCPConnector(limit=MAX_CONCURRENCY)
        async with aiohttp.ClientSession(connector=conn) as session:
            coros = [self._update_task(session, cycle, task) for task in tasks]
            temp = await gather_with_concurrency(
                100, coros, "Updating tasks with priorities"
            )

            for val in temp:
                if val:
                    failed.append(val)

        await asyncio.sleep(0.250)  # give time to close ssl connections
        return failed

    @handle_exception
    @check_learning_stage
    def update_tasks(self, cycle: int, tasks: List[Dict]) -> List[Dict]:
        """
        Update tasks with new score and labels.

        Return any tasks that experienced issues.
        """
        loop = asyncio.get_event_loop()
        temp = loop.run_until_complete(self._update_tasks(cycle, tasks))

        # update cycle
        self.context.learning.set_cycle_status(
            self.org_id, self.project_id, self.stage_name, cycle, "DONE"
        )
        return temp


class Learning2:
    """Reimplement Learning."""

    def __init__(
        self, context: RBContext, org_id: str, project_id: str, stage_name: str = ""
    ) -> None:
        """Construct Learning module."""
        self.context = context
        self.org_id = org_id
        self.project_id = project_id
        self.stage_name = stage_name

    @handle_exception
    @check_learning_stage
    def check_is_processing(self) -> bool:
        """Check if a job is in process already."""
        result = self.context.learning2.check_for_job(self.org_id, self.project_id)
        return bool(result.get("isProcessing"))

    @handle_exception
    @check_learning_stage
    def check_for_job(self, min_new_tasks: int = 100) -> bool:
        """Return true if there is a new job available."""
        result = self.context.learning2.check_for_job(self.org_id, self.project_id)
        if (
            result.get("newTasks", 0) >= min_new_tasks
            and result.get("isProcessing") is False
        ):
            return True
        if result.get("isProcessing") is True and (
            result.get("newTasks", 0) + result.get("newTasksProcessing", 0)
            >= min_new_tasks
        ):
            return True

        return False

    @handle_exception
    @check_learning_stage
    def get_learning_info(
        self, min_new_tasks: int = 100, concurrency: int = 100
    ) -> Dict:
        """
        Get a dictionary with lightly parsed redbrick response.

        The dictonary has fields:
            labeled
            unlabeled
            taxonomy
            type
        """
        if not self.check_for_job(min_new_tasks):
            raise Exception("Not enough new tasks since last training cycle.")

        taxonomy, td_type = self.context.learning2.get_taxonomy_and_type(
            self.org_id, self.project_id
        )

        def _filter_active_learning_tasks(
            org_id: str,
            project_id: str,
            stage_name: str,
            concurrency: int,
            cursor: Optional[str],
        ) -> Tuple[List[Dict], str]:
            entries, new_cursor = self.context.labeling.get_tasks_queue(
                org_id, project_id, stage_name, concurrency, cursor
            )
            return [
                item for item in entries if item["state"] == TaskStates.UNASSIGNED.value
            ], new_cursor

        my_queue_iter = PaginationIterator(
            partial(
                _filter_active_learning_tasks,
                self.org_id,
                self.project_id,
                self.stage_name,
                concurrency,
            )
        )
        tasks_in_queue = self.context.labeling.get_task_queue_count(
            self.org_id, self.project_id, self.stage_name, True
        )
        general_info = self.context.export.get_output_info(self.org_id, self.project_id)

        my_finished_iter = PaginationIterator(
            partial(
                self.context.export.get_datapoints_output,
                self.org_id,
                self.project_id,
                concurrency,
            )
        )

        def _parse_labeled_entry(item: Dict) -> Dict:
            items_presigned = item.get("itemsPresigned", []) or []
            name = item.get("name")
            items = item.get("items", []) or []
            task = item.get("task", {}) or {}
            task_id = task.get("taskId")
            label_data = item.get("labelData", {}) or {}
            labels = [
                clean_rb_label(label)
                for label in json.loads(label_data.get("labelsData", "[]") or "[]")
            ]

            return {
                "taskId": task_id,
                "items": items,
                "itemsPresigned": items_presigned,
                "name": name,
                "labels": labels,
                "labelsPath": label_data.get("labelsPath"),
            }

        def _parse_unlabeled_entry(item: Dict) -> Dict:
            datapoint = item.get("datapoint", {}) or {}
            items_presigned = datapoint.get("itemsPresigned", []) or []
            name = datapoint.get("name")
            items = datapoint.get("items", []) or []
            task_id = item.get("taskId")
            return {
                "taskId": task_id,
                "items": items,
                "status": item.get("state"),
                "itemsPresigned": items_presigned,
                "name": name,
            }

        print_info("Downloading unfinished tasks")
        unlabeled = [
            _parse_unlabeled_entry(item)
            for item in tqdm.tqdm(my_queue_iter, unit=" tasks", total=tasks_in_queue)
        ]

        print_info("Downloading finished tasks")
        labeled = [
            _parse_labeled_entry(item)
            for item in tqdm.tqdm(
                my_finished_iter,
                unit=" tasks",
                total=general_info["datapointCount"],
            )
        ]

        return {
            "labeled": labeled,
            "unlabeled": unlabeled,
            "taxonomy": taxonomy,
            "type": td_type,
        }

    async def _update_task2(
        self, session: aiohttp.ClientSession, task: Dict
    ) -> Optional[Dict]:
        """Attempt to update task."""
        try:
            task_input = deepcopy(task)
            if "labels" in task_input:
                task_input["labelsData"] = json.dumps(task_input["labels"])
                del task_input["labels"]

            labels_path: Optional[str] = None
            if "labelsPath" in task_input:
                labels_path = task_input["labelsPath"]
                del task_input["labelsPath"]
            if (
                labels_path
                and (labels_path.endswith(".nii") or labels_path.endswith(".nii.gz"))
                and os.path.isfile(labels_path)
            ):
                file_type = NIFTI_FILE_TYPES["nii"]
                presigned = await self.context.labeling.presign_labels_path(
                    session,
                    self.org_id,
                    self.project_id,
                    task_input["taskId"],
                    file_type,
                )
                if (
                    await upload_files(
                        [(labels_path, presigned["presignedUrl"], file_type)],
                        "Uploading labels",
                        False,
                    )
                )[0]:
                    task_input["labelsPath"] = presigned["filePath"]
            await self.context.learning2.update_prelabels_and_priorities(
                session, self.org_id, self.project_id, self.stage_name, [task_input]
            )
        except ValueError as error:
            print_error(error)
            point_error = deepcopy(task)
            point_error["error"] = error
            return point_error
        return None

    async def _update_tasks2(self, tasks: List[Dict]) -> List[Dict]:
        failed = []
        conn = aiohttp.TCPConnector(limit=MAX_CONCURRENCY)
        async with aiohttp.ClientSession(connector=conn) as session:
            coros = [self._update_task2(session, task) for task in tasks]
            temp = await gather_with_concurrency(
                30, coros, "Updating tasks with priorities"
            )

            for val in temp:
                if val:
                    failed.append(val)
        await asyncio.sleep(0.250)  # give time to close ssl connections
        return failed

    @handle_exception
    @check_learning_stage
    def update_tasks(self, cycle: int, tasks: List[Dict]) -> List[Dict]:
        """
        Update tasks with new score and labels.

        Return any tasks that experienced issues.
        """
        if cycle != 1:
            print_warning("Use of cycle field is deprecated")
        _ = cycle  # deprecated

        # backwards compatibility
        for task in tasks:
            if task.get("score"):
                task["priority"] = task["score"]
                del task["score"]

        loop = asyncio.get_event_loop()
        temp = loop.run_until_complete(self._update_tasks2(tasks))
        return temp

    @handle_exception
    @check_learning_stage
    def start_processing(self) -> None:
        """Signal to RedBrick AI that the training has begun."""
        self.context.learning2.start_processing(self.org_id, self.project_id)

    @handle_exception
    @check_learning_stage
    def end_processing(self) -> None:
        """Signal to RedBrick AI that the training has end."""
        self.context.learning2.end_processing(self.org_id, self.project_id)



"""Template creator for the most basic projects."""

from typing import List, Dict, Tuple

OUTPUT_NAME = "Output"
LABEL_NAME = "Label"
ACTIVE_LEARNING_ENTRY = "Finalized_Tasks"
MIN_REVIEW = 0
MAX_REVIEW = 6


def _get_middle_stages(reviews: int, passed_name: str) -> Tuple[List[Dict], str]:
    """Get label and review stages."""
    label_stage = {
        "brickName": "manual-labeling",
        "stageName": LABEL_NAME,
        "routing": {
            "nextStageName": "Review_1" if reviews > 0 else passed_name,
        },
        "stageConfig": {"isPrimaryStage": True},
    }
    feedback_stage = {
        "brickName": "feedback",
        "stageName": "Failed_Review",
        "routing": {
            "feedbackStageName": label_stage["stageName"],
        },
        "stageConfig": {},
    }

    stages = [label_stage]
    for i in range(1, reviews + 1):
        stages.append(
            {
                "brickName": "expert-review",
                "stageName": f"Review_{i}",
                "stageConfig": {},
                "routing": {
                    "passed": passed_name if i == reviews else f"Review_{i+1}",
                    "failed": feedback_stage["stageName"],
                },
            }
        )

    if reviews > 0:
        stages.append(feedback_stage)

    return stages, LABEL_NAME


def _get_active_learning_config(
    middle_stages: List[Dict], entry_point: str, batch_size: int, cycle_size: int
) -> List[Dict]:

    return (
        [
            {
                "brickName": "labelset-input",
                "routing": {
                    "nextStageName": "Active_Learning",
                },
                "stageName": "Input",
                "stageConfig": {},
            },
            {
                "brickName": "active-learning",
                "routing": {
                    "passed": OUTPUT_NAME,
                    "failed": entry_point,
                },
                "stageName": "Active_Learning",
                "stageConfig": {
                    "batchSize": batch_size,
                    "cycleSize": cycle_size,
                },
            },
            {
                "brickName": "labelset-output",
                "stageName": OUTPUT_NAME,
                "routing": {
                    "nextStageName": "END",
                },
                "stageConfig": {},
            },
        ]
        + middle_stages
        + [
            {
                "brickName": "feedback",
                "stageName": ACTIVE_LEARNING_ENTRY,
                "routing": {
                    "feedbackStageName": "Active_Learning",
                },
                "stageConfig": {},
            },
        ]
    )


def get_basic_project(reviews: int = 0) -> List[Dict]:
    """Get basic project config with reviews."""
    reviews = max(reviews, MIN_REVIEW)
    reviews = min(reviews, MAX_REVIEW)

    middle_stages, entry_point = _get_middle_stages(reviews, OUTPUT_NAME)

    input_stage = {
        "brickName": "labelset-input",
        "routing": {
            "nextStageName": entry_point,
        },
        "stageName": "Input",
        "stageConfig": {},
    }
    output_stage = {
        "brickName": "labelset-output",
        "stageName": OUTPUT_NAME,
        "routing": {
            "nextStageName": "END",
        },
        "stageConfig": {},
    }

    temp = [input_stage] + middle_stages + [output_stage]

    return temp


def get_active_learning_project(
    reviews: int, batch_size: int, cycle_size: int
) -> List[Dict]:
    """Get active learning project."""
    middle_stages, entry_point = _get_middle_stages(reviews, ACTIVE_LEARNING_ENTRY)

    return _get_active_learning_config(
        middle_stages, entry_point, batch_size, cycle_size
    )



"""CLI credentials handler."""
import os
from typing import Dict, List
from configparser import ConfigParser

from redbrick.common.context import RBContext


class CLICredentials:
    """CLICredentials entity."""

    _creds_file: str
    _creds: ConfigParser

    ENV_VAR: str = "REDBRICK_PROFILE"
    DEFAULT_PROFILE: str = "default"

    def __init__(self, creds_file: str) -> None:
        """Initialize CLICredentials."""
        self._creds_file = creds_file
        self._creds = ConfigParser()

        if self.exists:
            self._creds.read(self._creds_file)

    @property
    def exists(self) -> bool:
        """Boolean flag to indicate if credentials file exists."""
        if os.path.exists(self._creds_file):
            if os.path.isfile(self._creds_file):
                return True
            raise Exception(f"Not a file {self._creds_file}")
        return False

    @property
    def profile_names(self) -> List[str]:
        """List of all profiles."""
        return [
            section
            for section in self._creds.sections()
            if section != self.DEFAULT_PROFILE
        ]

    @property
    def selected_profile(self) -> str:
        """Get name of default selected profile."""
        assert self.exists, "Credentials file does not exist"
        profile = os.environ.get(self.ENV_VAR)
        if not profile:
            assert (
                self.DEFAULT_PROFILE in self._creds
                and "profile" in self._creds[self.DEFAULT_PROFILE]
            ), f"{self.DEFAULT_PROFILE} profile / {self.ENV_VAR} env not present"
            profile = self._creds[self.DEFAULT_PROFILE]["profile"]

        return profile

    @property
    def org_id(self) -> str:
        """Get org_id of default profile."""
        return self.get_profile(self.selected_profile)["org"].strip().lower()

    @property
    def context(self) -> RBContext:
        """Get SDK context."""
        return RBContext(
            api_key=self.get_profile(self.selected_profile)["key"].strip(),
            url=self.get_profile(self.selected_profile)["url"].strip().rstrip("/"),
        )

    def get_profile(self, profile_name: str) -> Dict[str, str]:
        """Get profile object from profile name."""
        assert (
            profile_name in self._creds
            and "key" in self._creds[profile_name]
            and "org" in self._creds[profile_name]
            and "url" in self._creds[profile_name]
        ), f"Profile not present / invalid profile in credentials : {profile_name}"
        return dict(self._creds[profile_name].items())

    def add_profile(
        self, profile_name: str, api_key: str, org_id: str, url: str
    ) -> None:
        """Add profile to credentials."""
        assert profile_name not in self.profile_names, "Profile already exists"
        self._creds[profile_name] = {
            "key": api_key.strip(),
            "org": org_id.strip().lower(),
            "url": url.strip().rstrip("/"),
        }

    def remove_profile(self, profile_name: str) -> None:
        """Remove profile from credentials."""
        assert profile_name in self.profile_names, "Profile does not exist"
        self._creds.pop(profile_name)

    def set_default(self, profile_name: str) -> None:
        """Set default profile in credentials."""
        assert profile_name in self.profile_names, "Profile does not exist"
        self._creds[self.DEFAULT_PROFILE] = {"profile": profile_name}

    def save(self) -> None:
        """Save credentials."""
        creds_dir = os.path.dirname(self._creds_file)
        if os.path.exists(creds_dir):
            if not os.path.isdir(creds_dir):
                raise Exception(f"Not a directory {creds_dir}")
        else:
            os.makedirs(creds_dir)

        profiles = self.profile_names
        if profiles:
            if (
                self.DEFAULT_PROFILE not in self._creds
                or "profile" not in self._creds[self.DEFAULT_PROFILE]
                or self._creds[self.DEFAULT_PROFILE]["profile"] not in profiles
            ):
                self.set_default(profiles[0])
            with open(self._creds_file, "w", encoding="utf-8") as creds:
                self._creds.write(creds)
        else:
            self.remove()

    def remove(self) -> None:
        """Remove credentials."""
        assert self.exists, "Credentials file does not exist"
        os.remove(self._creds_file)



"""Test CLI export."""
import contextlib
from datetime import datetime
import os
import json
import pickle
import shutil
import random
import subprocess
from uuid import uuid4
from typing import Generator, Optional, Tuple

import pytest
import tenacity

from redbrick.common.enums import LabelType, StorageMethod
from redbrick.project import RBProject


@contextlib.contextmanager
def create_project(
    org_id: str,
    api_key: str,
    url: str,
    label_type: LabelType,
    taxonomy: str = "Berkeley Deep Drive (BDD)",
    reviews: int = 1,
) -> Generator[Tuple[str, RBProject], None, None]:
    """Create project."""
    profile = datetime.strftime(datetime.now(), "%Y%m%d%H%M%S%f") + str(
        random.randint(0, 1000)
    )
    project_name = f"cli-{profile}"
    home_dir = os.path.expanduser("~")
    project_dir = os.path.join(home_dir, project_name)
    project: Optional[RBProject] = None
    try:
        os.makedirs(project_dir)
        subprocess.run(
            [
                "redbrick",
                "config",
                "add",
                "-o",
                org_id,
                "-k",
                api_key,
                "-u",
                url,
                "-p",
                profile,
            ],
            check=True,
        )
        subprocess.run(["redbrick", "config", "set", profile], check=True)
        subprocess.run(
            [
                "redbrick",
                "init",
                "-n",
                project_name,
                "-t",
                taxonomy,
                "-l",
                label_type.value,
                "-r",
                str(reviews),
                project_dir,
            ],
            check=True,
        )

        cache_name = os.listdir(os.path.join(project_dir, ".redbrick", "cache"))
        with open(
            os.path.join(
                project_dir, ".redbrick", "cache", cache_name[0], "project.pickle"
            ),
            "rb",
        ) as file_:
            project = pickle.load(file_)

        assert project
        yield project_dir, project
    finally:
        os.chdir(home_dir)
        shutil.rmtree(project_dir, ignore_errors=True)
        if project:
            project.context.project.delete_project(project.org_id, project.project_id)
        if os.environ.get("REDBRICK_SDK_SOURCE") == "GITHUB":
            subprocess.run(["redbrick", "config", "clear"], check=True)
        else:
            subprocess.run(["redbrick", "config", "remove", profile], check=True)


def upload_data(project: RBProject, num_tasks: int) -> None:
    """Upload data."""
    project.upload.create_datapoints(
        StorageMethod.PUBLIC,
        [
            {
                "name": str(uuid4()),
                "items": ["http://datasets.redbrickai.com/bccd/BloodImage_00000.jpg"],
            }
            for _ in range(num_tasks)
        ],
    )


def label_data(project: RBProject, num_tasks: int) -> None:
    """Label data."""
    categories = ["bus", "bike", "truck", "motor", "car", "train", "rider"]
    while num_tasks:
        for attempt in tenacity.Retrying(
            reraise=True,
            stop=tenacity.stop_after_attempt(10),
            wait=tenacity.wait_exponential(multiplier=1, min=1, max=10),
            retry=tenacity.retry_if_not_exception_type(
                (KeyboardInterrupt, PermissionError, ValueError)
            ),
        ):
            with attempt:
                tasks = project.labeling.get_tasks("Label", num_tasks)
                assert tasks
        num_tasks -= len(tasks)
        project.labeling.put_tasks(
            "Label",
            [
                {
                    **task,
                    "labels": [
                        {
                            "category": [["object", random.choice(categories)]],
                            "attributes": [],
                            "polygon": [
                                {
                                    "xnorm": random.random(),
                                    "ynorm": random.random(),
                                }
                                for _ in range(random.randint(3, 10))
                            ],
                            "labelid": str(uuid4()),
                        }
                        for _ in range(random.randint(1, 3))
                    ],
                }
                for task in tasks
            ],
        )


@pytest.mark.slow
def test_cli_export(org_id: str, api_key: str, url: str) -> None:
    """Test cli export."""
    with create_project(org_id, api_key, url, LabelType.IMAGE_POLYGON) as (
        project_dir,
        project,
    ):
        os.chdir(project_dir)

        total_tasks = 40
        upload_data(project, total_tasks)

        files = set(os.listdir(project_dir))
        task_count = [0, 1, total_tasks // 10, total_tasks // 5, total_tasks // 2]

        labeled = 0
        for count in task_count:
            label_data(project, count)
            labeled += count
            subprocess.run(["redbrick", "export"], check=True)
            new_files = set(os.listdir(project_dir))

            with open(
                os.path.join(project_dir, list(new_files - files)[0]),
                "r",
                encoding="utf-8",
            ) as file_:
                tasks = json.load(file_)

            assert sum([1 if task["labels"] else 0 for task in tasks]) == labeled

            files = new_files



"""CLI Input params."""

from redbrick.cli.input.api_key import CLIInputAPIKey
from redbrick.cli.input.number import CLIInputNumber
from redbrick.cli.input.profile import CLIInputProfile
from redbrick.cli.input.select import CLIInputSelect
from redbrick.cli.input.text import CLIInputText
from redbrick.cli.input.url import CLIInputURL
from redbrick.cli.input.uuid import CLIInputUUID



"""Input url handler."""
from typing import Optional

from InquirerPy import inquirer  # type: ignore

from redbrick.cli.cli_base import CLIInputParams


class CLIInputURL(CLIInputParams):
    """Input url handler."""

    def __init__(self, entity: Optional[str]) -> None:
        """Init handlers."""
        self.entity = entity
        self.error_message = "Invalid URL"

    def filtrator(self, entity: str) -> str:
        """Filter input entity."""
        return entity.strip().rstrip("/")

    def validator(self, entity: str) -> bool:
        """Validate input entity."""
        url = self.filtrator(entity)
        return url.startswith("http") and " " not in url and url.count("://") == 1

    def get(self) -> str:
        """Get filtered url value post validation."""
        self.entity = self.from_args()
        if self.entity is None:
            self.entity = inquirer.text(
                qmark=">",
                amark=">",
                message="URL:",
                default="https://api.redbrickai.com",
                transformer=self.filtrator,
                filter=self.filtrator,
                validate=self.validator,
                invalid_message=self.error_message,
            ).execute()
        return self.entity



"""Abstract interface to active learning API."""


from typing import Optional, List, Dict, Tuple
from abc import ABC, abstractmethod

import aiohttp


class LearningControllerInterface(ABC):
    """Abstract interface to Active Learning APIs."""

    @abstractmethod
    def check_for_job(
        self, org_id: str, project_id: str, stage_name: str
    ) -> Optional[int]:
        """Check for a job, returns cycle number."""

    @abstractmethod
    def get_batch_of_tasks(
        self,
        org_id: str,
        project_id: str,
        stage_name: str,
        first: int = 50,
        cursor: Optional[str] = None,
    ) -> Tuple[List[dict], Optional[str]]:
        """Get batch of tasks, paginated."""

    @abstractmethod
    def get_taxonomy_and_type(
        self, org_id: str, project_id: str, stage_name: str
    ) -> Tuple[dict, str]:
        """Get the taxonomy for active learning."""

    @abstractmethod
    def send_batch_learning_results(
        self,
        org_id: str,
        project_id: str,
        stage_name: str,
        cycle: int,
        tasks: List[Dict],
    ) -> None:
        """
        Send a batch of learning results.

        tasks is a list of dictionaries containing the following keys:
        {
            "taskId": "<>",
            "score": [0,1],
            "labels": { }  // see standard label format
        }
        """

    @abstractmethod
    async def send_batch_learning_results_async(
        self,
        aio_client: aiohttp.ClientSession,
        org_id: str,
        project_id: str,
        stage_name: str,
        cycle: int,
        tasks: List[Dict],
    ) -> None:
        """Perform send_batch_learning_results with asyncio."""

    @abstractmethod
    def set_cycle_status(
        self,
        org_id: str,
        project_id: str,
        stage_name: str,
        cycle: int,
        cycle_status: str,
    ) -> None:
        """Set status of current training cycle."""


class LearningController2Interface(ABC):
    """Abstract interface to Active Learning APIs."""

    @abstractmethod
    def check_for_job(self, org_id: str, project_id: str) -> Dict:
        """Check for a job, and num new tasks."""

    @abstractmethod
    def get_taxonomy_and_type(self, org_id: str, project_id: str) -> Tuple[dict, str]:
        """Get the taxonomy for active learning."""

    @abstractmethod
    async def update_prelabels_and_priorities(
        self,
        aio_client: aiohttp.ClientSession,
        org_id: str,
        project_id: str,
        stage_name: str,
        tasks: List[Dict],
    ) -> None:
        """Perform send_batch_learning_results with asyncio."""

    @abstractmethod
    def start_processing(self, org_id: str, project_id: str) -> None:
        """Set status of current training cycle."""

    @abstractmethod
    def end_processing(self, org_id: str, project_id: str) -> None:
        """Set status of current training cycle."""



"""Tests for bbox conversion."""


from redbrick.coco.bbox import rb2coco_bbox, coco2rb_bbox, is_coco_bbox


def test_rb2coco_bbox() -> None:
    """Test converting from rb bbox to coco bbox."""
    # arrange
    label = {
        "label_id": "1234",
        "category": [["object", "bus"]],
        "bbox2d": {"xnorm": 0, "ynorm": 0, "wnorm": 1, "hnorm": 1},
    }

    # action
    result = rb2coco_bbox(label, 1, 1, 2, 100, 100)
    result2 = coco2rb_bbox(result, [["object", "bux"]], 100, 100)
    result3 = rb2coco_bbox(result2, 1, 1, 2, 100, 100)
    # assert
    assert result == {
        "id": 1,
        "image_id": 1,
        "category_id": 2,
        "bbox": [0, 0, 100, 100],
        "area": 10000,
        "iscrowd": 0,
        "segmentation": [],
    }
    assert result == result3, "Conversion is not consistent/symmetric"
    assert is_coco_bbox(result)


def test_rb2coco_bbox_rounding_error() -> None:
    """
    Area calculation rounding error.

    Regression test for issue caused by rounding area at the end
    instead of using rounded width and height.
    """
    # arrange
    label = {
        "label_id": "1234",
        "category": [["object", "bus"]],
        "bbox2d": {"xnorm": 0, "ynorm": 0, "wnorm": 0.146, "hnorm": 0.116},
    }

    # action
    result = rb2coco_bbox(label, 1, 1, 2, 100, 100)

    # assert
    assert result["area"] == result["bbox"][2] * result["bbox"][3]


def test_is_coco_bbox() -> None:
    """Test if given coco label is an rb bbox."""
    assert is_coco_bbox(
        {
            "image_id": 1,
            "category_id": 2,
            "bbox": [0, 0, 100, 100],
            "area": 10000,
            "iscrowd": 0,
            "segmentation": [],
        }
    )
    assert not is_coco_bbox(
        {
            "image_id": 1,
            "category_id": 2,
            "bbox": [0, 0, 100, 100],
            "area": 10000,
            "iscrowd": 0,
            "segmentation": [1, 1, 1, 1],
        }
    )
    assert not is_coco_bbox(
        {
            "image_id": 1,
            "category_id": 2,
            "bbox": [0, 0, 100, 100],
            "area": 10000,
            "iscrowd": 0,
            "segmentation": [],
            "keypoints": [2, 1, 0],
        }
    )



"""CLI cache handler."""
import os
import shutil
import pickle
import zlib
import json
from typing import Any, Dict, List, Optional, Union
from datetime import datetime

from redbrick import __version__ as sdk_version
from redbrick.utils.common_utils import hash_sha256
from .conf import CLIConfiguration


class CLICache:
    """CLICache entity."""

    _cache_dir: str
    _conf: CLIConfiguration

    _cache_name: str
    _fixed_cache_name: str = "sdk-cache"

    CACHE_LIFETIME: int = 86400

    def __init__(self, cache_dir: str, conf: CLIConfiguration) -> None:
        """Initialize CLICache."""
        self._cache_dir = cache_dir
        self._conf = conf

        self._cache_name = "cache-" + sdk_version.split(".", 1)[0]

        if self._conf.exists and self._cache_name != self._conf.get_option(
            "cache", "name"
        ):
            self.clear_cache()
            self._conf.set_option("cache", "name", self._cache_name)
            self._conf.save()

    @property
    def exists(self) -> bool:
        """Boolean flag to indicate if cache directory exists."""
        if os.path.exists(self._cache_dir):
            if os.path.isdir(self._cache_dir):
                return True
            raise Exception(f"Not a directory {self._cache_dir}")
        return False

    def cache_path(self, *path: str, fixed_cache: bool = False) -> str:
        """Get cache file path."""
        path_dir = os.path.join(
            self._cache_dir,
            self._fixed_cache_name if fixed_cache else self._cache_name,
            *path[:-1],
        )
        os.makedirs(path_dir, exist_ok=True)
        return os.path.join(path_dir, path[-1])

    def get_object(self, entity: str) -> Any:
        """Get entity object from cache."""
        try:
            prev_timestamp = self._conf.get_option(entity, "refresh", "0")
            timestamp = int(datetime.utcnow().timestamp())
            prev_version = self._conf.get_option(entity, "version")
            if (
                prev_timestamp
                and prev_version
                and prev_version == sdk_version
                and timestamp - int(prev_timestamp) <= self.CACHE_LIFETIME
            ):
                cache_file = self.cache_path(f"{entity}.pickle")

                with open(cache_file, "rb") as cache:
                    return pickle.load(cache)
            return None
        except Exception:  # pylint: disable=broad-except
            return None

    def set_object(self, entity: str, obj: Any, save_conf: bool = True) -> None:
        """Set entity object into cache."""
        self._conf.set_option(
            entity, "refresh", str(int(datetime.utcnow().timestamp()))
        )
        self._conf.set_option(entity, "version", sdk_version)

        cache_file = self.cache_path(f"{entity}.pickle")

        with open(cache_file, "wb") as cache:
            pickle.dump(obj, cache)

        if save_conf:
            self._conf.save()

    def get_data(
        self,
        name: str,
        cache_hash: Optional[str],
        json_data: bool = True,
        fixed_cache: bool = False,
    ) -> Optional[Union[str, Dict, List]]:
        """Get cache data."""
        if cache_hash is None:
            return None
        cache_file = self.cache_path(name, fixed_cache=fixed_cache)
        if os.path.isfile(cache_file):
            with open(cache_file, "rb") as file_:
                data = file_.read()
            if cache_hash == hash_sha256(data):
                data = zlib.decompress(data)
                return json.loads(data) if json_data else data.decode()
        return None

    def set_data(
        self, name: str, entity: Union[str, Dict, List], fixed_cache: bool = False
    ) -> str:
        """Set cache data."""
        cache_file = self.cache_path(name, fixed_cache=fixed_cache)
        data = zlib.compress(
            (entity if isinstance(entity, str) else json.dumps(entity)).encode()
        )
        cache_hash = hash_sha256(data)
        with open(cache_file, "wb") as file_:
            file_.write(data)
        return cache_hash

    def clear_cache(self, all_caches: bool = False) -> None:
        """Clear project cache."""
        if not self.exists:
            return

        caches = list(
            filter(
                lambda cache: cache != self._fixed_cache_name,
                os.listdir(self._cache_dir),
            )
        )
        if not all_caches:
            caches = [cache for cache in caches if cache != self._cache_name]

        for cache in caches:
            shutil.rmtree(os.path.join(self._cache_dir, cache), ignore_errors=True)



"""Initialize repo module."""

from .export import ExportRepo
from .labeling import LabelingRepo
from .learning import LearningRepo, Learning2Repo
from .upload import UploadRepo
from .project import ProjectRepo



"""Input select handler."""
from typing import Any, List, Optional

from InquirerPy import inquirer  # type: ignore

from redbrick.cli.cli_base import CLIInputParams


class CLIInputSelect(CLIInputParams):
    """Input select handler."""

    def __init__(self, entity: Optional[str], name: str, options: List[Any]) -> None:
        """Init handlers."""
        self.entity = entity
        self.error_message = "Invalid " + name
        self.name = name
        self.options = options

    def filtrator(self, entity: str) -> str:
        """Filter input entity."""
        return entity.strip()

    def validator(self, entity: str) -> bool:
        """Validate input entity."""
        select = self.filtrator(entity)
        if not self.options:
            return False

        if isinstance(self.options[0], dict):
            return select in [val["name"] for val in self.options]

        return select in self.options

    def get(self) -> str:
        """Get filtered select value post validation."""
        self.entity = self.from_args()
        if self.entity is None:
            if self.options:
                self.entity = inquirer.fuzzy(
                    qmark=">", amark=">", message=self.name + ":", choices=self.options
                ).execute()
            else:
                raise ValueError(f"No {self.name} available")
        return self.entity



"""Input params handlers."""
from datetime import datetime
import re
from typing import List, Optional

from InquirerPy import inquirer  # type: ignore

from redbrick.cli.cli_base import CLIInputParams


class CLIInputProfile(CLIInputParams):
    """Input profile handler."""

    def __init__(
        self,
        entity: Optional[str],
        profiles: List[str],
        add: bool = False,
        default: Optional[str] = None,
    ) -> None:
        """Init handlers."""
        self.entity = entity
        self.error_message = (
            f"Non-alphanumeric / {'duplicate' if add else 'missing'} profile name"
        )
        self.add = add
        self.profiles = profiles
        self.default = default

    def filtrator(self, entity: str) -> str:
        """Filter input entity."""
        return entity.strip()

    def validator(self, entity: str) -> bool:
        """Validate input entity."""
        profile = self.filtrator(entity)
        return (
            profile.lower() != "default"
            and re.match(r"^\w+$", profile) is not None
            and (profile not in self.profiles if self.add else profile in self.profiles)
        )

    def get(self) -> str:
        """Get filtered profile value post validation."""
        self.entity = self.from_args()
        if self.entity is None:
            if self.add:
                self.entity = inquirer.text(
                    qmark=">",
                    amark=">",
                    message="Profile name:",
                    default="rb_" + datetime.strftime(datetime.now(), "%Y%m%d%H%M%S%f"),
                    transformer=self.filtrator,
                    filter=self.filtrator,
                    validate=self.validator,
                    invalid_message=self.error_message,
                ).execute()
            elif self.profiles:
                self.entity = inquirer.fuzzy(
                    qmark=">",
                    amark=">",
                    message="Profile name:",
                    choices=self.profiles,
                    default=self.default,
                ).execute()
            else:
                raise ValueError("No profiles available")
        return self.entity



"""Main object for RedBrick SDK."""

import json
from typing import Dict, List, Optional, Tuple

import tenacity

from redbrick.common.context import RBContext
from redbrick.common.enums import LabelType, StorageMethod
from redbrick.utils.logging import handle_exception, print_info


class RBProject:
    """
    Interact with a RedBrick project.

    Attributes
    -----------
    export: redbrick.export.Export
        Interface for managing exporting data and
        labels from your redbrick ai projects.

    labeling: redbrick.labeling.Labeling
        Interface for programmatically labeling your
        redbrick ai tasks.

    review: redbrick.labeling.Labeling
        Interface for programmatically reviewing your
        redbrick ai tasks.

    upload: redbrick.upload.Upload
        Interface for programmatically managing upload
        of your data and labels.
    """

    def __init__(self, context: RBContext, org_id: str, project_id: str) -> None:
        """Construct RBProject."""
        # pylint: disable=import-outside-toplevel
        from redbrick.upload import Upload
        from redbrick.learning import Learning, Learning2
        from redbrick.labeling import Labeling
        from redbrick.export import Export

        self.context = context

        self._org_id = org_id
        self._project_id = project_id
        self._project_name: str
        self._stages: List[Dict]
        self._td_type: str
        self._taxonomy_name: str
        self._project_url: str
        self._label_storage: Optional[Tuple[str, str]] = None

        # check if project exists on backend to validate
        self._get_project()

        self.upload = Upload(context, org_id, project_id, self.project_type)

        self.output_stage_name: str = "Output"
        self.learning = Learning(self.context, self.org_id, self.project_id)
        self.learning2 = Learning2(self.context, self.org_id, self.project_id)
        for stage in self._stages:
            if stage["brickName"] == "manual-labeling":
                if json.loads(stage["stageConfig"]).get("isPrimaryStage"):
                    self.learning2 = Learning2(
                        self.context, self.org_id, self.project_id, stage["stageName"]
                    )
            elif stage["brickName"] == "active-learning":
                self.learning = Learning(
                    self.context, self.org_id, self.project_id, stage["stageName"]
                )
            elif stage["brickName"] == "labelset-output":
                self.output_stage_name = stage["stageName"]

        self.labeling = Labeling(context, org_id, project_id)
        self.review = Labeling(context, org_id, project_id, review=True)
        self.export = Export(
            context, org_id, project_id, self.project_type, self.output_stage_name
        )

    @property
    def org_id(self) -> str:
        """
        Read only property.

        Retrieves the unique Organization UUID that this project belongs to
        """
        return self._org_id

    @property
    def project_id(self) -> str:
        """
        Read only property.

        Retrieves the unique Project ID UUID.
        """
        return self._project_id

    @property
    def name(self) -> str:
        """
        Read only name property.

        Retrieves the project name.
        """
        return self._project_name

    @property
    def url(self) -> str:
        """
        Read only property.

        Retrieves the project URL.
        """
        return self._project_url

    @property
    def taxonomy_name(self) -> str:
        """
        Read only taxonomy_name property.

        Retrieves the taxonomy name.
        """
        return self._taxonomy_name

    @property
    def project_type(self) -> LabelType:
        """
        Read only project_type property.

        Retrieves the type of the project.
        """
        return LabelType(self._td_type)

    @property
    def label_storage(self) -> Tuple[str, str]:
        """
        Read only label_storage property.

        Retrieves the label storage id and path.
        """
        if not self._label_storage:
            self._label_storage = self.context.project.get_label_storage(
                self.org_id, self.project_id
            )
        return self._label_storage

    def __wait_for_project_to_finish_creating(self) -> Dict:
        try:
            for attempt in tenacity.Retrying(
                reraise=True,
                stop=tenacity.stop_after_attempt(10),
                wait=tenacity.wait_exponential(multiplier=1, min=1, max=10),
                retry=tenacity.retry_if_not_exception_type(
                    (KeyboardInterrupt, PermissionError, ValueError)
                ),
            ):
                with attempt:
                    project = self.context.project.get_project(
                        self.org_id, self.project_id
                    )
                    if project["status"] == "CREATING":
                        if attempt.retry_state.attempt_number == 1:
                            print_info("Project is still creating...")
                        raise Exception("Unknown problem occurred")
        except tenacity.RetryError as error:
            raise Exception("Unknown problem occurred") from error

        if project["status"] == "REMOVING":
            raise Exception("Project has been deleted")
        if project["status"] == "CREATION_FAILURE":
            raise Exception("Project failed to be created")
        if project["status"] == "CREATION_SUCCESS":
            return project
        raise Exception("Unknown problem occurred")

    def _get_project(self) -> None:
        """Get project to confirm it exists."""
        project = self.__wait_for_project_to_finish_creating()

        self._project_name = project["name"]
        self._td_type = project["tdType"]
        self._taxonomy_name = project["taxonomy"]["name"]
        self._stages = self.context.project.get_stages(self.org_id, self.project_id)
        self._project_url = project["projectUrl"]

    def __str__(self) -> str:
        """Get string representation of RBProject object."""
        return f"RedBrick Project - {self.name} - id:( {self.project_id} )"

    def __repr__(self) -> str:
        """Representation of object."""
        return str(self)

    @handle_exception
    def set_label_storage(self, storage_id: str, path: str) -> Tuple[str, str]:
        """Set label storage method for a project."""
        path = (
            f"{self.org_id}/{self.project_id}"
            if storage_id == StorageMethod.REDBRICK
            else path.rstrip("/")
        )
        self.context.project.set_label_storage(
            self.org_id, self.project_id, storage_id, path
        )
        self._label_storage = (storage_id, path)
        return self.label_storage



"""Setup the python module."""
from setuptools import setup, find_packages  # type: ignore

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

# Extract from file as py36 does not support setuptools cfg metadata version
with open("redbrick/__init__.py", "r", encoding="utf-8") as fh:
    lines = fh.readlines()
    for line in lines:
        if line.strip().replace(" ", "").startswith("__version__="):
            version = line.split("=")[1].split("#")[0].strip().strip("\"'")
            break
    else:
        raise Exception("Could not find version")

install_requires = [
    "requests==2.23.0",
    "tqdm==4.50.0",
    "aiohttp==3.7.4",
    "cchardet==2.1.7",
    "aiodns==3.0.0",
    "Pillow==8.4.0; python_version=='3.6'",
    "Pillow==9.0.1; python_version>'3.6'",
    "shapely==1.8.0",
    "numpy>=1.15",
    "matplotlib>=3.2",
    "scikit-image>=0.17.2, <=0.18.3",
    "pyparsing==2.4.7",
    "aiofiles==0.8.0",
    "rasterio==1.2.10; sys_platform=='darwin'",
    "rasterio==1.2.10; sys_platform=='linux'",
    "InquirerPy>=0.2.1, <=0.3.0",
    "halo==0.0.31",
    "nest-asyncio==1.5.4",
    "tenacity==8.0.1",
    "natsort==8.0.2",
    "nibabel==3.2.1",
    "boto3==1.20.43",
]

setup(
    name="redbrick-sdk",
    version=version,
    url="https://github.com/redbrick-ai/redbrick-sdk",
    description="RedBrick platform Python SDK!",
    py_modules=["redbrick"],
    python_requires=">=3.6, <3.10",
    packages=find_packages(),
    long_description=long_description,
    long_description_content_type="text/markdown",
    include_package_data=True,
    install_requires=install_requires,
    entry_points={
        "console_scripts": ["redbrick=redbrick.cli:cli_main"],
    },
    extras_require={
        "dev": [
            "twine==3.1.1",
            "wheel==0.35.1",
            "pytest-mock==3.3.1",
            "pytest==6.1.2",
            "black==22.3.0",
            "mypy==0.790",
            "mypy-extensions==0.4.3",
            "pylint==2.11.1",
            "pycodestyle==2.6.0",
            "pydocstyle==5.1.1",
            "flake8==3.8.4",
            "flake8-print==3.1.4",
            "eradicate==1.0",
            "pytest-cov>=2.8.1",
            "pytest-asyncio==0.12.0",
            "pytest-eradicate==0.0.5",
            "pytest-pycodestyle==2.2.0",
            "pytest-pydocstyle==2.2.0",
            "pytest-black==0.3.12",
            "pytest-pylint==0.17.0",
            "pytest-mypy==0.7.0",
            "pytest-flake8==1.0.6",
            "pytest-randomly==3.4.1",
        ]
    },
)



"""Abstract interface to exporting data from a project."""

from typing import Optional, List, Dict, Tuple
from abc import ABC, abstractmethod
from datetime import datetime

import aiohttp


class ExportControllerInterface(ABC):
    """Abstract interface to define methods for Export."""

    @abstractmethod
    def get_output_info(
        self,
        org_id: str,
        project_id: str,
    ) -> Dict:
        """Get info about the output labelset and taxonomy."""

    @abstractmethod
    def get_datapoints_output(
        self,
        org_id: str,
        project_id: str,
        first: int = 50,
        cursor: Optional[str] = None,
    ) -> Tuple[List[Dict], Optional[str]]:
        """Get datapoints that have made it to the output of the project."""

    @abstractmethod
    def datapoints_in_project(
        self, org_id: str, project_id: str, stage_name: Optional[str] = None
    ) -> int:
        """Get number of datapoints in project."""

    @abstractmethod
    def get_datapoints_latest(
        self,
        org_id: str,
        project_id: str,
        stage_name: Optional[str] = None,
        cache_time: Optional[datetime] = None,
        first: int = 50,
        cursor: Optional[str] = None,
    ) -> Tuple[List[Dict], Optional[str], Optional[datetime]]:
        """Get the latest datapoints."""

    @abstractmethod
    def get_datapoint_latest(self, org_id: str, project_id: str, task_id: str) -> Dict:
        """Get the latest labels for a single datapoint."""

    @abstractmethod
    async def get_labels(
        self, session: aiohttp.ClientSession, org_id: str, project_id: str, dp_id: str
    ) -> Dict:
        """Get input labels."""

    @abstractmethod
    def task_search(
        self,
        org_id: str,
        project_id: str,
        stage_name: Optional[str] = None,
        task_search: Optional[str] = None,
        first: int = 50,
        after: Optional[str] = None,
    ) -> Tuple[List[Dict], Optional[str]]:
        """Task search."""
